package it.unibo.tw0809.ui;

import it.unibo.tw0809.xml.DTD_DOM_Validator;
import it.unibo.tw0809.xml.DTD_SAX_Validator;
import it.unibo.tw0809.xml.IValidator;
import it.unibo.tw0809.xml.XSD_DOM_Validator;
import it.unibo.tw0809.xml.XSD_SAX_Validator;

public class ArgsParser {

	public static final String XML = "xml";
	public static final String DTD = "dtd";
	public static final String XSD = "xsd";
	public static final String SAX = "sax";
	public static final String DOM = "dom";

	// --------------------------------------------

	private StringBuffer message = new StringBuffer();

	private String[] args;

	// --------------------------------------------

	public ArgsParser(String[] args) {
		this.args = args;
	}

	// --------------------------------------------

	public String getMessage() {
		return this.message.toString();
	}

	public boolean checkArgs() {
		if ( args.length != 3 ) {
			message.append("Program expects 3 arguments:");
			message.append("\n");
			message.append(" 1) the absolute name of the XML file");
			message.append("\n");
			message.append(" 2) the name of XSD/DTD file to validate against");
			message.append("\n");
			message.append(" 3) the validation method to use (sax/dom)");
			message.append("\n");
			message.append("Note: XSD filename must be relative to that of the XML one in case of DOM-based validaton");
			return false;
		}
		for ( String arg : args ) {
			if ( arg == null ) return false;
		}
		if ( ! args[0].endsWith(XML) ) {
			message.append("Argument #1 must be an 'xml' file");
			return false;
		}
		if (
			! args[1].endsWith(DTD) 
				&& 
			! args[1].endsWith(XSD)
		) {
			message.append("Argument #2 must be a 'dtd' or 'xsd' file");
			return false;
		}
		if (
			! args[2].equalsIgnoreCase(DOM) 
				&& 
			! args[2].equalsIgnoreCase(SAX)
		) {
			message.append("Argument #3 must be 'sax' or 'dom'");
			return false;
		}
		return true;
	}
	
	public IValidator getValidator() {

		boolean dtd = false;
		boolean xsd = false;
		
		boolean sax = false;
		boolean dom = false;

		if ( args[1].endsWith(DTD) )
			dtd = true;
		if ( args[1].endsWith(XSD) )
			xsd = true;
		if ( args[2].equalsIgnoreCase(SAX) )
			sax = true;
		if ( args[2].equalsIgnoreCase(DOM) )
			dom = true;

		IValidator validator = null;
		
		if ( dtd && sax )
			validator = new DTD_SAX_Validator();
		else if ( dtd && dom )
			validator = new DTD_DOM_Validator();
		else if ( xsd && sax )
			validator = new XSD_SAX_Validator();
		else /* if ( xsd && dom ) */
			validator = new XSD_DOM_Validator();
		
		return validator;

	}

}
