package it.unibo.tw0809.xml;
import it.unibo.tw0809.util.FileUtil;
import it.unibo.tw0809.util.MyHandler;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

/**
 * This example found on
 * http://www.onjava.com/pub/a/onjava/2004/09/15/schema-validation.html
 * 
 * @author Deepak Vohra
 * 
 * modified by Pisi
 */
public class XSD_DOM_Validator 
implements IValidator {

	public static String default_xml_filename = "validation/prova.xml";
	public static String default_xsd_filename = "prova.xsd";

	// ---------------------------------------

	private boolean verbose = false;

	private StringBuffer result = new StringBuffer();
	
	// ---------------------------------------

	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	public String getResult() {
		return this.result.toString();
	}

	public boolean validate(String xml_filename, String relative_xsd_filename) {

		try {

			// Another way to validate an XML document is with a JAXP DocumentBuilder. To begin, import the DocumentBuilderFactory and DocumentBuilder classes. The DocumentBuilder class is used to obtain a org.w3c.dom.Document document from an XML document, while the DocumentBuilderFactory class is used to obtain a DocumentBuilder parser.
	
			//		To validate with a DocumentBuilder parser, set the System property javax.xml.parsers.DocumentBuilderFactory:
			//
			//		System.setProperty("javax.xml.parsers.DocumentBuilderFactory",
			//		               "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
	
			// 1. Next, you need to create a DocumentBuilderFactory.
			// An instance of DocumentBuilderFactory is found by applying the following rules and taking the first one that succeeds:
			//   a. Use the javax.xml.parsers.DocumentBuilderFactory system property.
			//   b. Use the properties file lib/jaxp.properties in the JRE directory.
			//   c. Use the META-INF/services/javax.xml.parsers.DocumentBuilderFactory file with the Services API.
			//   d. Use the Platform default DocumentBuilderFactory instance.
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	
			// 2a. To parse a XML document with a namespace, set the setNamespaceAware() feature to true. By default, the setNamespaceAware() feature is set to false.
			factory.setNamespaceAware(true);
			// 2b. Set the setValidating() feature of the DocumentBuilderFactory to true to make the parser a validating parser. By default, the setValidating() feature is set to false.
			factory.setValidating(true);
			// 2c. Set the schemaLanguage and schemaSource attributes of the DocumentBuilderFactory. The schemaLanguage attribute specifies the schema language for validation. The schemaSource attribute specifies the XML schema document to be used for validation.
			factory.setAttribute(
			    "http://java.sun.com/xml/jaxp/properties/schemaLanguage",
			    "http://www.w3.org/2001/XMLSchema");
			factory.setAttribute(
			    "http://java.sun.com/xml/jaxp/properties/schemaSource",
			    relative_xsd_filename);
	
			// 3. Create a DocumentBuilder parser.
			// This returns a new DocumentBuilder, with the parameters configured in the DocumentBuilderFactory. 
			DocumentBuilder builder = factory.newDocumentBuilder();
	
			// 3.5 Check everything's in place
			if ( verbose ) FileUtil.printFileContent(relative_xsd_filename);
			if ( verbose ) FileUtil.printFileContent(xml_filename);
			
			// 4. Parse
			MyHandler myHandler = new MyHandler(this.result);
			builder.setErrorHandler(myHandler);
			builder.parse(xml_filename);
			
			// 5. Result returning
			boolean success = myHandler.success(); 
			if ( success )
			    this.result.append(xml_filename + " is valid.");
			else
			    this.result.insert(0, xml_filename + " is not valid because of").append("\n");
			return success;

		}
		catch (Exception e) {
			this.result = new StringBuffer();
			this.result.append(e.getClass().getCanonicalName()).append("\n").append(e.getMessage());
		    return false;
		}
	
	}

}
