package it.unibo.tw0809.xml;
import it.unibo.tw0809.util.FileUtil;

import java.io.File;

import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.xml.sax.SAXException;

/**
 * This example found on
 * http://www.ibm.com/developerworks/xml/library/x-javaxmlvalidapi.html
 *
 * @author Elliotte Rusty Harold (elharo@metalab.unc.edu), Adjunct Professor, Polytechnic University
 * 
 * modified by Pisi
 */
public class XSD_SAX_Validator
implements IValidator {

	public static String default_xml_filename = "validation/prova.xml";
	public static String default_xsd_filename = "validation/prova.xsd";

	// ---------------------------------------

	private boolean verbose = true;

	private StringBuffer result = new StringBuffer();
	
	// ---------------------------------------

	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	public String getResult() {
		return this.result.toString();
	}

	public boolean validate(String xml_filename, String absolute_xsd_filename) {
		
		try {
			// 1. Lookup a factory for the W3C XML Schema language
			SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
			// SchemaFactory factory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");	SAME AS ABOVE
			
			// SINCE JDK 1.4, DTD VALIDATION IS DEFINED BUT NOT IMPLEMENTED!
			// SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.XML_DTD_NS_URI);			IF USING OBSOLETE DTD INSTEAD OF XSD 
			
			// 2. Compile the schema. 
			// Here the schema is loaded from a java.io.File, but you could use 
			// a java.net.URL or a javax.xml.transform.Source instead.
			if ( verbose ) 
				FileUtil.printFileContent(absolute_xsd_filename);
			File schemaLocation = new File(absolute_xsd_filename);
			Schema schema = factory.newSchema(schemaLocation);
			
			// 3. Get a validator from the schema.
			Validator validator = schema.newValidator();
			
			// 4. Parse the document you want to check.
			if ( verbose ) 
				FileUtil.printFileContent(xml_filename);
			File xmlDocument = new File(xml_filename);
			Source source = new StreamSource(xmlDocument);
			
			// 5. Check the document
			try {
			    validator.validate(source);
			    this.result.append(xml_filename + " is valid.");
			    return true;
			}
			catch (SAXException ex) {
			    this.result.append(xml_filename + " is not valid because of").append("\n").append(ex.getMessage());
			    return false;
			}  
			
		}
		catch (Exception e) {
			this.result = new StringBuffer();
			this.result.append(e.getClass().getCanonicalName()).append("\n").append(e.getMessage());
		    return false;
		}

	}
		
}
